/***********************************************************************
 * SPDX-FileCopyrightText: 2003-2004 Max Howell <max.howell@methylblue.com>
 * SPDX-FileCopyrightText: 2008-2013 Martin Sandsmark <martin.sandsmark@kde.org>
 *
 * SPDX-License-Identifier: GPL-2.0-only OR GPL-3.0-only OR LicenseRef-KDE-Accepted-GPL
 ***********************************************************************/

#pragma once

#include "Config.h"
#include "version.h"

#undef VERSION
#undef PACKAGE
#undef PRETTYNAME

#define APP_NAME "filelight"
#define APP_VERSION FILELIGHT_VERSION_STRING
#define APP_PRETTYNAME "Filelight"
